module PgnTests exposing (testBangScenarioPropTestsRevealed, testParseTurnInlineComments, testParseTurnNoComments, testParseTurnsEolComments)

import Expect
import Pgn
import Test exposing (Test, test)


testParseTurnNoComments : Test
testParseTurnNoComments =
    let
        turn =
            "11. Bd2 11... Ne3"

        res =
            { number = "11", white = "Bd2", black = "Ne3" }
    in
    test "Test that a turn without comments parses correctly" <|
        \_ ->
            case Pgn.parseMove turn of
                Ok t ->
                    Expect.equal t res

                Err err ->
                    err
                        |> Pgn.parseErrorToString turn
                        |> Expect.fail


testParseTurnInlineComments : Test
testParseTurnInlineComments =
    let
        turn =
            "11. {[some comments] [and more]} Bd2 {another set} 11... {even more} Ne3 {lastly, these}"

        res =
            { black = "Ne3", number = "11", white = "Bd2" }
    in
    test "Test that a turn with inline comments parses correctly" <|
        \_ ->
            case Pgn.parseMove turn of
                Ok t ->
                    Expect.equal t res

                Err err ->
                    err
                        |> Pgn.parseErrorToString turn
                        |> Expect.fail


testParseTurnsEolComments : Test
testParseTurnsEolComments =
    let
        turns =
            "11. Bd2 11... Ne3; comment on turn 11\n12. Qe2 12... Nc2+; comment on turn 12\n13. Kf1 13... Nxa1; comment on turn 13\n"

        res =
            [ { black = "Ne3"
              , number = "11"
              , white = "Bd2"
              }
            , { black = "Nc2+"
              , number = "12"
              , white = "Qe2"
              }
            , { black = "Nxa1"
              , number = "13"
              , white = "Kf1"
              }
            ]
    in
    test "Test that a turn with EOL comments parses correctly" <|
        \_ ->
            case Pgn.parseMoves turns of
                Ok t ->
                    Expect.equal t res

                Err err ->
                    err
                        |> Pgn.parseErrorToString turns
                        |> Expect.fail


testBangScenarioPropTestsRevealed : Test
testBangScenarioPropTestsRevealed =
    let
        tagpair =
            "[! \"!\"]"

        res =
            { title = "!"
            , value = "!"
            }
    in
    test "Test an exclamation point in a tag pair parsed correcrtly" <|
        \_ ->
            case Pgn.parseTagPair tagpair of
                Ok t ->
                    Expect.equal t res

                Err err ->
                    err
                        |> Pgn.parseErrorToString tagpair
                        |> Expect.fail
